#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from swiftclient import client
from swiftclient.client import ClientException


class DataStoreException(Exception):
    pass


class DataStore(object):

    DataStoreException = DataStoreException

    def __init__(self, component, auth_config, identifier=None, public=False):
        self.container = self._get_container(component, identifier)
        self.auth_config = auth_config

        self._setup_auth()

        self._create_container(self.container)
        self.change_visibility(public=public)
        self.container_url = self._get_container_url()

    def list_files(self):
        hdr, data = client.get_container(*self.auth_bits)

        replace_text = '{}.'.format(self.container)
        return [x['name'].replace(replace_text, '') for x in data]

    def put_file(self, filename, contents):
        name = self._get_file_name(filename)
        try:
            client.put_object(*self.auth_bits, name=name, contents=contents)
        except ClientException as e:
            raise self.DataStoreException(
                "Failed to upload file: {}, Error: {}".format(filename, e)
            )

        return self.file_path(filename)

    def get_file(self, filename, with_headers=False):
        name = self._get_file_name(filename)

        contents = None
        try:
            hdr, contents = client.get_object(*self.auth_bits, name=name)
        except ClientException as e:
            raise self.DataStoreException(
                "Failed to get file: {}, Error: {}".format(filename, e)
            )

        if with_headers:
            return hdr, contents

        return contents

    def change_visibility(self, public=False):
        read_acl = ''

        if public:
            read_acl = '.r:*'

        args = self.auth_bits
        args += ({'X-Container-Read': read_acl},)
        client.post_container(*args)

    def delete_file(self, filename):
        name = self._get_file_name(filename)
        try:
            client.delete_object(*self.auth_bits, name=name)
        except ClientException as e:
            raise self.DataStoreException(
                "Failed to delete file: {}, Error: {}".format(filename, e)
            )

    def clear(self):
        try:
            files = client.get_container(*self.auth_bits)[1]

            for file in files:
                try:
                    client.delete_object(*self.auth_bits, name=file['name'])
                except ClientException as e:
                    raise self.DataStoreException(
                        "Failed to delete file: {}".format(e)
                    )
        except ClientException as e:
            raise self.DataStoreException(
                "Failed to get container: {}".format(e)
            )

    def delete(self, recursive=False):
        try:
            if recursive:
                self.clear()

            client.delete_container(*self.auth_bits)
        except ClientException as e:
            raise self.DataStoreException(
                "Failed to delete container: {}".format(e)
            )

    def file_path(self, filename):
        name = self._get_file_name(filename)

        return "{}/{}".format(self.container_url, name)

    # Private functions
    def _has_file(self, filename):
        name = self._get_file_name(filename)
        result = True

        try:
            client.head_object(*self.auth_bits, name=name)
        except ClientException:
            result = False

        return result

    def _get_container(self, component, identifier=None):
        container = component

        if identifier is not None and identifier != "":
            container = "{}.{}".format(container, identifier)

        return container

    def _setup_auth(self):
        config = self.auth_config

        try:
            self.url, self.token = client.get_auth(
                config.get('auth_url', ''),
                config.get('auth_user', ''),
                config.get('auth_password', ''),
                auth_version=2,
                tenant_name=config.get('auth_tenant_name', '')
            )
        except client.ClientException:
            self.url = None
            self.token = None

        if self.url is None or self.token is None:
            raise self.DataStoreException(
                "Missing or invalid authentication info."
            )

        self.auth_bits = (self.url, self.token, self.container)

    def _create_container(self, container):
        client.put_container(self.url, self.token, container)

    def _get_file_name(self, filename):
        return "{}.{}".format(self.container, filename)

    def _get_container_url(self):
        return "{}/{}".format(self.url, self.container)
