#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ci_utils import data_store
import urllib2
import unittest
import yaml


class TestDataStore(unittest.TestCase):

    def setUp(self):

        with open('/tmp/auth_config.yaml') as fp:
            self.auth_config = yaml.safe_load(fp)

        self.ds = data_store.DataStore("test", auth_config=self.auth_config)
        self.assert_(self.ds, "Failed to create DataStore")
        self.addCleanup(self.ds.delete, recursive=True)

        self.ds.clear()
        self.filename = 'test_file'
        self.contents = 'some data'

        super(TestDataStore, self).setUp()

    def test_list_files_empty(self):
        """ Test listing files on an empty container. """
        files = self.ds.list_files()
        self.assertEqual([], files)

    def test_add_file(self):
        """ Test adding a file. """

        self.ds.put_file(self.filename, self.contents)
        files = self.ds.list_files()
        self.assertEqual([self.filename], files)

    def test_add_file_contents(self):
        """ Test adding a file has correct contents. """

        self.ds.put_file(self.filename, self.contents)
        contents = self.ds.get_file(self.filename)

        self.assertEqual(contents, self.contents)

    def test_public_container(self):
        """ Test accesssing a public container. """

        self.ds.put_file(self.filename, self.contents)
        self.ds.change_visibility(public=True)

        response = urllib2.urlopen(self.ds.file_path(self.filename))
        self.assertEqual(response.code, 200)

    def test_delete_file(self):
        """ Test delete file removes the file. """

        self.ds.put_file(self.filename, self.contents)
        files = self.ds.list_files()
        self.assertEqual([self.filename], files)

        self.ds.delete_file(self.filename)
        files = self.ds.list_files()
        self.assertEqual([], files)

    def test_auth_missing(self):
        """ Test that missing auth details is handled correctly. """

        with self.assertRaises(data_store.DataStoreException):
            data_store.DataStore("test", auth_config={})

    def test_put_file_link(self):
        """ Test put file returns a valid link. """

        link = self.ds.put_file(self.filename, self.contents)
        self.ds.change_visibility(public=True)
        resp = urllib2.urlopen(link)
        self.ds.delete_file(self.filename)
        self.assertEqual(200, resp.getcode())


if __name__ == "__main__":
    unittest.main()
