#!/usr/bin/python

import argparse
import os
import re
import string
import subprocess


def _get_args():
    parser = argparse.ArgumentParser(description='Modify a cloud image')
    parser.add_argument(
        "-c", "--chroot", required=True,
        help="Path for chroot where image is mounted")
    parser.add_argument(
        "-r", "--repository", required=True, action='append',
        help="List of PPAs to add")
    parser.add_argument(
        "-p", "--package", required=True, action='append',
        help="List of extra packages to install")
    parser.add_argument(
        "-s", "--series", required=True,
        help="Series of Ubuntu the image is based on")

    args = parser.parse_args()
    return args


def _parse_ppas(chroot, ppalist, series):
    aptlines = []
    for ppa in ppalist:
        ppamatch = re.match("ppa:(.+)/(.+)", ppa)
        #This will throw an exception for malformed ppas, intentional for now
        (team, repo) = ppamatch.groups()
        aptlines.append(
            "deb-src http://ppa.launchpad.net/{team}/{repo}/ubuntu "
            "{series} main".format(team=team, repo=repo, series=series))
    return aptlines


def _chrooted(chroot, cmd):
    subprocess.check_output(['chroot', chroot] + cmd.split(" "))


def _run_chroot_cmds(chroot, ppalist, pkglist):
    '''Run commands in the chroot to do update the sources and add packages
    '''
    #Build the list of PPAs for the sources.list file
    with open('%s/etc/sources.list.d/ci.list' % chroot, 'a+') as f:
        for debline in ppalist:
            f.write('\n%s' % debline)

    _chrooted(chroot, "/usr/bin/apt-get update")
    _chrooted(chroot, "/usr/bin/apt-get install -y {pkglist}".format(
              pkglist=string.join(pkglist, " ")))
    _chrooted(chroot, "/usr/bin/apt-get clean")

    #create a big null data file and remove it to make it more compressable
    zerofile = '%s/zero.tmp' % chroot
    filldata = '\0'*4096
    with open(zerofile, 'wb') as f:
        while True:
            try:
                f.write(filldata)
            except IOError:
                break
    os.unlink(zerofile)


def modify_image(chroot, repos, series, packages):
    '''Add packges and PPAs to a cloud image mounted in a chroot'''
    ppalist = _parse_ppas(chroot, repos, series)
    _run_chroot_cmds(chroot, ppalist, packages)


def main():
    args = _get_args()
    return modify_image(args.chroot, args.repository, args.series,
                        args.package)

if __name__ == '__main__':
    main()
