#!/usr/bin/env python
# Ubuntu CI Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import logging
import json
import os
import sys
import urllib2

import yaml

# the worker might not have installed this module, so determine the path
# and add it, so we can always safely import stuff
sys.path.append(os.path.join(os.path.dirname(__file__), '../../ci-utils'))
from ci_utils import amqp_utils

format_list = {'json', 'yaml'}
# Make up some resonable data returned from each service
data = {
    'master': {
        'autopilot': {
            'version': '1.3.1+13.10.20131003.1-0ubuntu2~fginther.1',
            'files': [
                '/path/to/changes',
                '/path/to/dsc',
                '/path/to/diff.gz']}},
    'image_builder': {
        'result': 'PASSED',
        'image': '/path/to/image'},
    'test_runner': {
        'result': 'PASSED',
        'artifacts': [
            '/path/to/artifact1',
            '/path/to/artifact2']},
    'publisher': {
        'result': 'PASSED',
        'image': '/path/to/image'},
}


# set up a root logger for everything
stdout_handler = logging.StreamHandler(stream=sys.stdout)
formatter = logging.Formatter(
    '%(asctime)s %(levelname)-5s %(name)s: %(message)s', datefmt='%H:%M:%S')
stdout_handler.setFormatter(formatter)
l = logging.getLogger('')
l.addHandler(stdout_handler)
l.setLevel(logging.DEBUG)

logger = logging.getLogger('lander_service_wrapper')


def _drain_progress(amqp_config, queue):
    class MessageSync(object):
        progress = logging.getLogger('PROGRESS_TRIGGER')

        def on_message(self, msg):
            data = json.loads(msg.body)

            # print "state" as a label for the rest of the items in the list
            params = {x: y for (x, y) in data.iteritems() if x != 'state'}
            if params:
                self.progress.info('%s: %s', data['state'], params)
            else:
                self.progress.info(data['state'])

            # TODO update ticket-system

            msg.channel.basic_ack(msg.delivery_tag)
            if data.get('exit'):
                self.data = data
                # this will kick us out of the run-forever worker loop
                msg.channel.basic_cancel(msg.consumer_tag)

    sync = MessageSync()
    amqp_utils.process_queue(amqp_config, queue, sync.on_message, delete=True)
    return sync.data


def _post(url, data):
    req = urllib2.Request(url)
    req.add_header('Content-Type', 'application/json')
    resp = urllib2.urlopen(req, json.dumps(data))
    if resp.getcode() == 204:
        return None  # 204 is an empty post reply
    return resp.headers['location']


def _get(url):
    resp = urllib2.urlopen(url)
    resp = resp.read()
    return json.loads(resp)


def _handle_ppa_assigner(args):
    config = _get_config(args)
    url = '%s/api/v1/ppa/' % config['master']['ppa_assigner_url']
    loc = _post(url, {'ticket_id': config['master']['request_id']})
    data = _get(loc)
    result = {
        'result': 'PASSED',
        'ppa': data['name'],
    }
    return result


def _handle_bsbuilder(args):
    config = _get_config(args)
    url = '%s/api/v1/build_source' % config['master']['bsb_url']

    queue = '%s-bsbuilder' % config['master']['progress_trigger']

    params = {
        'source_packages': config['master']['request_parameters'],
        'ppa': config['ppa_assigner']['ppa'],
        'progress_trigger': queue,
    }

    _post(url, params)

    # Its a little silly to start *after* the _post. However, this allows
    # the code to not have to deal with _post raising an exception and our
    # code having to stop the thread in the middle of an amqp wait logic
    logger.info('starting progress handler...')
    return _drain_progress(args.amqp_config, queue)


def _handle_image_builder(args):
    # TODO implement
    return data['image_builder']


def _handle_publisher(args):
    # TODO implement
    return data['publisher']


def _handle_test_runner(args):
    # TODO implement
    return data['test_runner']


def _get_config(args):
    with open(args.input_file) as f:
        if args.input_format == 'json':
            config = json.loads(f.read())
        else:
            config = yaml.safe_load(f)
    return config


def _get_service_handlers():
    '''get the availble service handlers.

    This is a handy mechanism so we don't have to maintain a list of available
    service handlers. Any function prefixed with '_handle_' will be exposed to
    the user.
    '''
    return [x[8:] for x in globals() if x.startswith('_handle_')]


def _get_parser():
    parser = argparse.ArgumentParser(
        description='Wraps the REST API calls with a progress queue monitor.')
    parser.add_argument('--service',
                        required=True,
                        choices=_get_service_handlers(),
                        help='The service name to call.')
    parser.add_argument('--input-file',
                        required=True,
                        help='File containing all of the input parameters.')
    parser.add_argument('--input-format',
                        required=False,
                        choices=format_list,
                        default='json')
    parser.add_argument('--output-file',
                        required=True,
                        help='File to store the API call results.')
    parser.add_argument('--output-format',
                        required=False,
                        choices=format_list,
                        default='json')
    return parser


def main(args):
    args.amqp_config = amqp_utils.get_config()
    if not args.amqp_config:
        return 1  # the get_config code prints an error

    handler = globals()['_handle_' + args.service]
    logger.debug('Executing service wrapper: {}'.format(args.service))

    rc = 1
    try:
        results = handler(args)
        rc = 0
    except urllib2.HTTPError as e:
        body = e.read()
        logging.exception('Unable to handle request: %s: %s', e, body)
        results = {
            'result': 'ERROR',
            'resp_code': e.reason,
            'resp_body': body,
        }
    except Exception as e:
        logging.exception('Unexpected error: %s', e)
        results = {
            'result': 'FAILED',
            'error': e.message,
        }

    with open(args.output_file, 'w') as output_file:
        out_data = None
        if args.output_format == 'json':
            out_data = json.dumps(results)
        elif args.output_format == 'yaml':
            out_data = yaml.dump(results)
        output_file.write('{}\n'.format(out_data))

    return rc


if __name__ == '__main__':
    args = _get_parser().parse_args()
    exit(main(args))
