# Ubuntu CI Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import os
import StringIO
import sys
import tempfile
import unittest
import urllib2

import mock

sys.path.append(os.path.join(os.path.dirname(__file__), '../../bin'))
import lander_service_wrapper


class TestWrapper(unittest.TestCase):
    '''Sanity tests of the lander_service_wrapper.py script.'''

    def setUp(self):
        super(TestWrapper, self).setUp()
        patch = mock.patch('ci_utils.amqp_utils.get_config')
        patch.start()
        self.addCleanup(patch.stop)

        self.results = tempfile.mkstemp(prefix='lander-test-')[1]
        self.addCleanup(os.unlink, self.results)

        self.params = tempfile.mkstemp(prefix='lander-test-')[1]
        self.addCleanup(os.unlink, self.params)
        data = {
            'master': {
                'request_id': 1,
                'request_parameters': 'blah',
                'ppa_assigner_url': 'foo',
                'bsb_url': 'bar',
                'progress_trigger': 'progress-trigger',
            },
            'ppa_assigner': {
                'ppa': 'ppa-name',
            }
        }
        with open(self.params, 'w') as f:
            f.write(json.dumps(data))

        self.base_args = [
            '--input-file', self.params, '--output-file', self.results]

    def testBadService(self):
        '''make sure we exit gracefully for a bad service name'''
        args = self.base_args + ['--service', 'bad-name']
        with self.assertRaises(SystemExit):
            lander_service_wrapper._get_parser().parse_args(args)

    @mock.patch('lander_service_wrapper._post')
    def testPPAAssigner404(self, post):
        '''Ensure a "no ppas" type error generates a response'''
        err_msg = 'foo bar'
        post.side_effect = urllib2.HTTPError(
            'url', 404, '404', None, StringIO.StringIO(err_msg))

        args = self.base_args + ['--service', 'ppa_assigner']
        args = lander_service_wrapper._get_parser().parse_args(args)
        rc = lander_service_wrapper.main(args)
        self.assertEqual(1, rc)

        with open(self.results) as f:
            data = json.loads(f.read())
            self.assertEqual('ERROR', data['result'])
            self.assertEqual(err_msg, data['resp_body'])
            self.assertEqual('404', data['resp_code'])

    @mock.patch('lander_service_wrapper._post')
    def testPPAAssignerUnexpected(self, post):
        '''Ensure unexpected errors still generate a response'''
        err_msg = 'foo bar'
        post.side_effect = RuntimeError(err_msg)

        args = self.base_args + ['--service', 'ppa_assigner']
        args = lander_service_wrapper._get_parser().parse_args(args)
        rc = lander_service_wrapper.main(args)
        self.assertEqual(1, rc)

        with open(self.results) as f:
            data = json.loads(f.read())
            self.assertEqual('FAILED', data['result'])
            self.assertEqual(err_msg, data['error'])

    @mock.patch('lander_service_wrapper._post')
    @mock.patch('lander_service_wrapper._get')
    def testPPAAssignerSucceeds(self, get, post):
        '''Ensure we generate the proper response if things succeed.'''
        ppa = 'ppa:foo'
        post.return_value = 'http://foo/api/v1/ppa/1'
        get.return_value = {'name': ppa}

        args = self.base_args + ['--service', 'ppa_assigner']
        args = lander_service_wrapper._get_parser().parse_args(args)
        rc = lander_service_wrapper.main(args)
        self.assertEqual(0, rc)

        with open(self.results) as f:
            data = json.loads(f.read())
            self.assertEqual('PASSED', data['result'])
            self.assertEqual(ppa, data['ppa'])

    @mock.patch('lander_service_wrapper._post')
    def testBSBuilderUnexpected(self, post):
        '''Ensure unexpected errors still generate a response'''
        err_msg = 'foo bar'
        post.side_effect = RuntimeError(err_msg)

        args = self.base_args + ['--service', 'bsbuilder']
        args = lander_service_wrapper._get_parser().parse_args(args)
        rc = lander_service_wrapper.main(args)
        self.assertEqual(1, rc)

        with open(self.results) as f:
            data = json.loads(f.read())
            self.assertEqual('FAILED', data['result'])
            self.assertEqual(err_msg, data['error'])

    @mock.patch('lander_service_wrapper._post')
    @mock.patch('ci_utils.amqp_utils.connection')
    def testBSBuilderSucceeds(self, connection, post):
        '''Ensure we generate the proper response if things succeed.'''

        # mock the run-forever logic to ensure we return what we expect
        state = {'state': 'DONE', 'foo': 'bar', 'exit': True}

        def amqp():
            def run_forever(channel, queue, callback):
                m = mock.Mock()
                m.body = json.dumps({'state': 'WAITING'})
                callback(m)
                m.body = json.dumps(state)
                callback(m)
            return run_forever

        args = self.base_args + ['--service', 'bsbuilder']
        args = lander_service_wrapper._get_parser().parse_args(args)

        with mock.patch('ci_utils.amqp_utils._run_forever', new_callable=amqp):
            rc = lander_service_wrapper.main(args)
            self.assertEqual(0, rc)

        with open(self.results) as f:
            data = json.loads(f.read())
            self.assertEqual(state, data)
