# Ubuntu CI Engine
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import logging
import urllib2

from django.conf import settings
from django.db import models

from ppa_assigner import launchpad


class PPA(models.Model):
    AVAILABLE = 0
    RESERVED = 10
    DIRTY = 20
    CLEANING = 30
    STATES = (
        (AVAILABLE, 'available'),
        (RESERVED, 'reserved'),
        (DIRTY, 'dirty'),
        (CLEANING, 'cleaning'),
    )
    name = models.CharField(max_length=4096, unique=True)

    state = models.PositiveIntegerField(choices=STATES, default=AVAILABLE)
    ticket_id = models.PositiveIntegerField()
    timestamp = models.DateTimeField(auto_now_add=True)

    def __unicode__(self):
        val = '%s (%s)' % (self.name, self._get_state_str())
        if self.state == PPA.RESERVED:
            val += ' ticket=%s' % self.ticket_id
        return val

    def _get_state_str(self):
        for state, label in PPA.STATES:
            if self.state == state:
                return label
        raise RuntimeError('Unsupported state: %s' % self.state)

    @staticmethod
    def _merge_ppas(lp_ppas):
        model_ppas = set([x.name for x in PPA.objects.all()])
        add = lp_ppas - model_ppas
        remove = model_ppas - lp_ppas

        logging.info('adding new ppas: %r', add)
        for x in add:
            PPA.objects.create(name=x, ticket_id=0)

        logging.info('removing ppas: %r', add)
        for x in remove:
            ppa = PPA.objects.get(name=x)
            if ppa.state == PPA.RESERVED:
                # log something, but we may as well delete. because if the
                # ppa doesn't exist in LP we are already in a catastrophic
                # situation and shouldn't let any other user re-use this
                logging.error('ppa %s appears to be in use', x)
            ppa.delete()

    @staticmethod
    def populate_from_launchpad():
        '''Get list of PPAs from launchpad and ensure they exist here.'''
        url = settings.LAUNCHPAD_API_BASE + '/~%s/ppas'
        url = url % settings.LAUNCHPAD_PPA_USER
        logging.info('populating ppa model from: %s', url)
        try:
            ppas = ['ppa:%s/%s' % (settings.LAUNCHPAD_PPA_USER, x['name'])
                    for x in launchpad.lp_collection(url)]
            PPA._merge_ppas(set(ppas))
        except urllib2.URLError as e:
            logging.exception('Could not populate PPAs from launchpad', e)
            raise

    @staticmethod
    def reserve(ticket_id):
        # "select_for_update" provides proper db-level locking so we
        # don't accidentally have 2 concurrent requests find and reserve the
        # same PPA
        qs = PPA.objects.select_for_update().filter(state=PPA.AVAILABLE)
        if qs.count() == 0:
            raise PPA.DoesNotExist('No free PPAs exist in pool')
        pk = qs[0].pk
        now = datetime.datetime.utcnow()
        qs.filter(pk__in=(pk,)).update(
            state=PPA.RESERVED, ticket_id=ticket_id, timestamp=now)
        return PPA.objects.get(id=pk)
