# Ubuntu Continuous Integration Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tastypie import fields
from tastypie.constants import ALL
from tastypie.authorization import Authorization
from tastypie.resources import ModelResource
from ticket.models import (Ticket, SubTicket, SourcePackageUpload, Artifact,
                           SubTicketWorkflowStep, SubTicketWorkflowStepStatus,
                           TicketWorkflowStep, TicketWorkflowStepStatus,
                           get_enum_title)

from project.api import SourcePackageResource


class TicketTranslatedResource(ModelResource):

    def dehydrate_current_workflow_step(self, bundle):
        return get_enum_title(bundle.data["current_workflow_step"],
                              TicketWorkflowStep)

    def dehydrate_status(self, bundle):
        return get_enum_title(bundle.data["status"], TicketWorkflowStepStatus)


class SubTicketTranslatedResource(ModelResource):

    def dehydrate_current_workflow_step(self, bundle):
        return get_enum_title(bundle.data["current_workflow_step"],
                              SubTicketWorkflowStep)

    def dehydrate_status(self, bundle):
        return get_enum_title(bundle.data["status"],
                              SubTicketWorkflowStepStatus)


class TicketResource(TicketTranslatedResource):

    class Meta:
        queryset = Ticket.objects.all()
        allowed_methods = ['get', 'post']
        authorization = Authorization()


class SourcePackageUploadResource(ModelResource):
    sourcepackage = fields.ToOneField(SourcePackageResource, 'sourcepackage',
                                      full=True)

    class Meta:
        queryset = SourcePackageUpload.objects.all()
        allowed_methods = ['get', 'post']
        authorization = Authorization()
        resource_name = 'spu'


class SubTicketResource(SubTicketTranslatedResource):
    ticket = fields.ToOneField(TicketResource, 'ticket', full=True)
    source_package_upload = fields.ToOneField(SourcePackageUploadResource,
                                              'source_package_upload',
                                              full=True)

    class Meta:
        queryset = SubTicket.objects.all()
        allowed_methods = ['get', 'post']
        authorization = Authorization()


class ArtifactResource(ModelResource):
    subticket = fields.ToOneField(SubTicketResource, 'subticket', full=True)

    class Meta:
        queryset = Artifact.objects.all()
        allowed_methods = ['get', 'post']
        authorization = Authorization()


class FullArtifactResource(ModelResource):

    class Meta:
        queryset = Artifact.objects.all()
        allowed_methods = ['get']


class FullSubTicketResource(SubTicketTranslatedResource):
    source_package_upload = fields.ToOneField(SourcePackageUploadResource,
                                              'source_package_upload',
                                              full=True)
    artifact = fields.ToManyField(FullArtifactResource, 'artifact_set',
                                  full=True)

    class Meta:
        queryset = SubTicket.objects.all()
        allowed_methods = ['get']


class FullTicketResource(TicketTranslatedResource):
    subticket = fields.ToManyField(FullSubTicketResource, 'subticket_set',
                                   full=True)

    class Meta:
        queryset = Ticket.objects.all()
        allowed_methods = ['get']


class TicketUpdateStatusResource(TicketTranslatedResource):

    class Meta:
        queryset = Ticket.objects.all()
        fields = ['id', 'current_workflow_step', 'status']
        allowed_methods = ['get', 'patch']
        authorization = Authorization()
        resource_name = 'updatestatus'
        filtering = {
            "id": ('exact'),
        }


class TicketStatusResource(TicketTranslatedResource):

    class Meta:
        queryset = Ticket.objects.all()
        fields = ['id', 'title', 'current_workflow_step', 'status', 'created',
                  'updated']
        allowed_methods = ['get']
        resource_name = 'status'
        filtering = {
            "current_workflow_step": ALL,
        }
