# Ubuntu Continuous Integration Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from model_mommy import mommy
from ci_utils.tastypie.test import TastypieTestCase
from ticket.models import Ticket, SourcePackageUpload, SubTicket, Artifact


class APICreateTicketResourceTest(TastypieTestCase):

    def setUp(self):
        super(APICreateTicketResourceTest, self).setUp('/api/v1')
        self.ticket = mommy.make('Ticket')
        self.resource = 'ticket/'
        self.detail_url = self.resource + '{0}/'.format(self.ticket.pk)
        self.post_ticket_data = {
            'owner': 'owner@example.com',
            'title': 'My first ticket!',
            'description': 'This if my first ticket. See what it can do',
            'bug_id': '12345',
            'added_binaries': 'binary-1,binary-2',
            'removed_binaries': 'binary-3,binary-4',
        }

    def test_post_ticket(self):
        # Check how many are there first.
        self.assertEqual(Ticket.objects.count(), 1)
        self.post(resource=self.resource, params=self.post_ticket_data)
        # Verify a new one has been added.
        self.assertEqual(Ticket.objects.count(), 2)

    def test_patch_detail(self):
        # Grab the current data & modify it slightly.
        original_data = self.getResource(self.detail_url)
        new_data = original_data.copy()
        new_data['owner'] = 'ci@example.com'

        self.assertEqual(Ticket.objects.count(), 1)
        resp = self.client.patch('/api/v1/' + self.detail_url, data=new_data)
        self.assertHttpMethodNotAllowed(resp)
        # Make sure the count hasn't changed & we did an update.
        self.assertEqual(Ticket.objects.count(), 1)
        # Check for updated data.
        self.assertEqual(Ticket.objects.get(pk=self.ticket.pk).owner,
                         self.ticket.owner)

    def test_delete_ticket_not_allowed(self):
        resp = self.delete(resource=self.detail_url)
        self.assertHttpMethodNotAllowed(resp)


class APICreateSPUResourceTest(TastypieTestCase):

    def setUp(self):
        super(APICreateSPUResourceTest, self).setUp('/api/v1')
        self.sourcepackage = mommy.make('SourcePackage')
        self.spu = mommy.make('SourcePackageUpload',
                              sourcepackage=self.sourcepackage)
        self.resource = 'spu/'
        self.detail_url = self.resource + '{0}/'.format(self.spu.pk)
        self.sourcepackage_uri = '/api/v1/sourcepackage/{0}/'.format(
            self.sourcepackage.pk)
        self.post_spu_data = {
            'sourcepackage': self.sourcepackage_uri,
            'version': '1.0',
        }

    def test_post_spu(self):
        # Check how many are there first.
        self.assertEqual(SourcePackageUpload.objects.count(), 1)
        self.post(resource=self.resource, params=self.post_spu_data)
        # Verify a new one has been added.
        self.assertEqual(SourcePackageUpload.objects.count(), 2)

    def test_patch_detail(self):
        # Grab the current data & modify it slightly.
        original_data = self.getResource(self.detail_url)
        new_data = original_data.copy()
        new_data['version'] = '2.0'

        self.assertEqual(SourcePackageUpload.objects.count(), 1)
        resp = self.client.patch('/api/v1/' + self.detail_url, data=new_data)
        self.assertHttpMethodNotAllowed(resp)
        # Make sure the count hasn't changed & we did an update.
        self.assertEqual(SourcePackageUpload.objects.count(), 1)
        # Check for updated data.
        self.assertEqual(SourcePackageUpload.objects.get(
                         pk=self.spu.pk).version, self.spu.version)

    def test_delete_spu_not_allowed(self):
        resp = self.delete(resource=self.detail_url)
        self.assertHttpMethodNotAllowed(resp)


class APICreateSubTicketResourceTest(TastypieTestCase):

    def setUp(self):
        super(APICreateSubTicketResourceTest, self).setUp('/api/v1')
        self.ticket = mommy.make('Ticket')
        self.spu = mommy.make('SourcePackageUpload')
        self.subticket = mommy.make('SubTicket', ticket=self.ticket,
                                    source_package_upload=self.spu)
        self.resource = 'subticket/'
        self.detail_url = self.resource + '{0}/'.format(self.subticket.pk)
        self.ticket_uri = '/api/v1/spu/{0}/'.format(self.ticket.pk)
        self.spu_uri = '/api/v1/spu/{0}/'.format(self.spu.pk)
        self.post_subticket_data = {
            'source_package_upload': self.spu_uri,
            'ticket': self.ticket_uri,
            'assignee': 'test@example.com',
        }

    def test_post_subticket(self):
        # Check how many are there first.
        self.assertEqual(SubTicket.objects.count(), 1)
        self.post(resource=self.resource, params=self.post_subticket_data)
        # Verify a new one has been added.
        self.assertEqual(SubTicket.objects.count(), 2)

    def test_patch_detail(self):
        # Grab the current data & modify it slightly.
        original_data = self.getResource(self.detail_url)
        new_data = original_data.copy()
        new_data['assignee'] = 'me@example.com'

        self.assertEqual(SubTicket.objects.count(), 1)
        resp = self.client.patch('/api/v1/' + self.detail_url, data=new_data)
        self.assertHttpMethodNotAllowed(resp)
        # Make sure the count hasn't changed & we did an update.
        self.assertEqual(SubTicket.objects.count(), 1)
        # Check for updated data.
        self.assertEqual(SubTicket.objects.get(pk=self.subticket.pk).assignee,
                         self.subticket.assignee)

    def test_delete_subticket_not_allowed(self):
        resp = self.delete(resource=self.detail_url)
        self.assertHttpMethodNotAllowed(resp)


class APICreateArtifactResourceTest(TastypieTestCase):

    def setUp(self):
        super(APICreateArtifactResourceTest, self).setUp('/api/v1')
        self.subticket = mommy.make('SubTicket')
        self.artifact = mommy.make('Artifact', subticket=self.subticket)
        self.resource = 'artifact/'
        self.detail_url = self.resource + '{0}/'.format(self.artifact.pk)
        self.subticket_uri = '/api/v1/subticket/{0}/'.format(self.subticket.pk)
        self.post_artifact_data = {
            'type': 'SPU',
            'name': 'my_artifact',
            'subticket': self.subticket_uri,
            'reference': 'm1CTLg5FHY',
        }

    def test_post_artifact(self):
        # Check how many are there first.
        self.assertEqual(Artifact.objects.count(), 1)
        self.post(resource=self.resource, params=self.post_artifact_data)
        # Verify a new one has been added.
        self.assertEqual(Artifact.objects.count(), 2)

    def test_patch_detail(self):
        # Grab the current data & modify it slightly.
        original_data = self.getResource(self.detail_url)
        new_data = original_data.copy()
        new_data['name'] = 'my_upload'

        self.assertEqual(Artifact.objects.count(), 1)
        resp = self.client.patch('/api/v1/' + self.detail_url, data=new_data)
        self.assertHttpMethodNotAllowed(resp)
        # Make sure the count hasn't changed & we did an update.
        self.assertEqual(Artifact.objects.count(), 1)
        # Check for updated data.
        self.assertEqual(Artifact.objects.get(pk=self.artifact.pk).name,
                         self.artifact.name)

    def test_delete_artifact_not_allowed(self):
        resp = self.delete(resource=self.detail_url)
        self.assertHttpMethodNotAllowed(resp)


class APIUpdateTicketStatuses(TastypieTestCase):

    def setUp(self):
        super(APIUpdateTicketStatuses, self).setUp('/api/v1')
        self.ticket = mommy.make('Ticket')
        self.ticket_detail_url = 'updatestatus/{0}/'.format(self.ticket.pk)

    def test_patch_ticket_status(self):
        new_data = {'current_workflow_step': '200', 'status': '210'}
        self.assertEqual(Ticket.objects.count(), 1)
        self.patch('/api/v1/' + self.ticket_detail_url,
                   new_data)
        # Make sure the count hasn't changed & we did an update.
        self.assertEqual(Ticket.objects.count(), 1)
        # Check for updated data.
        self.assertEqual(Ticket.objects.get(
                         pk=self.ticket.pk).current_workflow_step, 200)
        self.assertEqual(Ticket.objects.get(pk=self.ticket.pk).status, 210)
